@@ -981,17 +981,18 @@ namespace HtmlAgilityPack
 
 		public void PauseLoad()
 		{
+			Debug ("Pausing load");
 			_pause_parsing = true;
 		}
 
 		public void ResumeLoad()
 		{
-			if (! _pause_parsing || _parserState._lastquote == -1)
-				throw new Exception ("Load() was not paused previously");
-
 			if (_done_parsing)
 				return;
 
+			if (! _pause_parsing || _parserState._lastquote == -1)
+				throw new Exception ("Load() was not paused previously");
+
 			// Reset the old states and values
 			int _lastquote = _parserState._lastquote;
 			_parserState._lastquote = -1;
@@ -1365,6 +1366,8 @@ namespace HtmlAgilityPack
 			QuotedAttributeValue,
 			ServerSideCode,
 			PcDataQuote,
+			PcDataCommentSingleLine,
+			PcDataCommentMultiLine,
 			PcData
 		}
 
@@ -1440,6 +1443,7 @@ namespace HtmlAgilityPack
 			while (! _pause_parsing && ! _stop_parsing && ! _text.Eof (_index))
 			{
 				_c = _text[_index];
+				Debug (String.Format ("_index : {0}({2})({1}) ", _index, (char)_c, _state));
 				IncrementPosition();
 
 				switch(_state)
@@ -1719,12 +1723,29 @@ namespace HtmlAgilityPack
 						}
 						break;
 
+					// handle // abcd A's webpage
+					case ParseState.PcDataCommentSingleLine:
+						if (_c == '\n')
+							_state = ParseState.PcData;
+						break;
+
+					// handle /* multiple lines */
+					case ParseState.PcDataCommentMultiLine:
+						if (_c == '/' && _text [_index - 2] == '*')
+							_state = ParseState.PcData;
+						break;
 					case ParseState.PcData:
-						Debug ("PCDATA " + _currentnode.Name + " " + _text.Substring(_index-1,  _currentnode._namelength+2));
+						Debug (String.Format ("PCDATA ({0}) {1} {2}", _index, _currentnode.Name, _text.Substring(_index-1,  _currentnode._namelength+2)));
 						if (_c == '\"' || _c == '\''){
 							_pcdata_quote_char = _c;
 							_state = ParseState.PcDataQuote;
 							break;
+						} else if (_c == '/' && _text [_index - 2] == '/') {
+							_state = ParseState.PcDataCommentSingleLine;
+							break;
+						} else if (_c == '*' && _text [_index - 2] == '/') {
+							_state = ParseState.PcDataCommentMultiLine;
+							break;
 						}
 						// look for </tag + 1 char
 
