@@ -33,6 +33,7 @@ using System.Diagnostics;
 namespace Beagle.Util {
 
 	public enum LogLevel {
+		Always,
 		Error,
 		Warn,
 		Debug,
@@ -145,7 +146,7 @@ namespace Beagle.Util {
 					log_date = DateTime.ParseExact (date, "yyyy-MM-dd-HH-mm-ss", null);
 					if (log_date < magic_date)
 						file.Delete ();
-				} catch (Exception e) {	}
+				} catch { }
 			}				
 		}
 
@@ -199,7 +200,7 @@ namespace Beagle.Util {
 
 		static private void WriteLine (LogLevel level, string format, object [] args, Exception ex) 
 		{
-			if (cutoff_level < level)
+			if (level != LogLevel.Always && cutoff_level < level)
 				return;
 
 			string ex_str = null;
@@ -226,7 +227,7 @@ namespace Beagle.Util {
 			StringBuilder prefix_builder;
 			prefix_builder = new StringBuilder ();
 			prefix_builder.Append ('\n'); // start w/ a newline
-			prefix_builder.AppendFormat ("{0:yyMMdd HHmmssffff} {1:00000} ",
+			prefix_builder.AppendFormat ("{0:yyyyMMdd HH:mm:ss.ffff} {1:00000} ",
 						     DateTime.Now, Process.GetCurrentProcess ().Id);
 			prefix_builder.Append (program_identifier_truncated);
 
@@ -241,6 +242,9 @@ namespace Beagle.Util {
 			case LogLevel.Debug:
 				prefix_builder.Append ("DEBUG");
 				break;
+			case LogLevel.Always:
+				prefix_builder.Append (" INFO");
+				break;
 			default:
 				prefix_builder.Append (" HUH?");
 				break;
@@ -338,6 +342,21 @@ namespace Beagle.Util {
 		{
 			WriteLine (LogLevel.Error, null, null, ex);
 		}
+
+		static public void Always (string message, params object [] args)
+		{
+			WriteLine (LogLevel.Always, message, args, null);
+		}
+		
+		static public void Always (Exception ex, string message, params object [] args)
+		{
+			WriteLine (LogLevel.Always, message, args, ex);
+		}
+
+		static public void Always (Exception ex)
+		{
+			WriteLine (LogLevel.Always, null, null, ex);
+		}
 	}
 }
 
