@@ -87,10 +87,21 @@ namespace Beagle.Util {
 
                 static public DateTime StringToDateTime (string str)
                 {
-			if (str == null || str == String.Empty)
+			if (String.IsNullOrEmpty (str) || str.Length != 14)
 				return new DateTime ();
 
-			return DateTime.ParseExact (str, TimeFormat, CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal);
+			int year, month, day, hour, min, sec;
+
+			// Pretty simple format, parse it ourself
+			year  = int.Parse (str.Substring (0, 4));
+			month = int.Parse (str.Substring (4, 2));
+			day   = int.Parse (str.Substring (6, 2));
+			hour  = int.Parse (str.Substring (8, 2));
+			min   = int.Parse (str.Substring (10, 2));
+			sec   = int.Parse (str.Substring (12, 2));
+
+			DateTime dt = new DateTime (year, month, day, hour, min, sec, DateTimeKind.Utc);
+			return dt;
                 }
 
 		static public string DateTimeToFuzzy (DateTime dt)
