@@ -40,6 +40,8 @@ using LNS = Lucene.Net.Search;
 
 using Beagle;
 using Beagle.Util;
+
+using Stopwatch = Beagle.Util.Stopwatch;
 using FSQ = Beagle.Daemon.FileSystemQueryable.FileSystemQueryable;
 
 namespace Beagle.Daemon 
@@ -47,11 +49,13 @@ namespace Beagle.Daemon
 	class BuildIndex 
 	{
 		static string [] argv;
-
-		static bool arg_recursive = false, arg_delete = false, arg_debug = false, arg_cache_text = false, arg_disable_filtering = false, arg_disable_restart = false, arg_disable_directories = false;
-
 		static Hashtable remap_table = new Hashtable ();
 
+		static bool arg_recursive, arg_delete, arg_debug;
+		static bool arg_cache_text, arg_disable_filtering;
+		static bool arg_disable_restart, arg_disable_directories;
+		static bool arg_disable_on_battery;
+
 		static string arg_output, arg_tag, arg_source;
 
 		/////////////////////////////////////////////////////////
@@ -161,6 +165,7 @@ namespace Beagle.Daemon
 					++i;
 					break;
 				*/
+
 				case "--target":
 					if (next_arg != null)
 						arg_output = Path.IsPathRooted (next_arg) ? next_arg : Path.GetFullPath (next_arg);
@@ -171,6 +176,10 @@ namespace Beagle.Daemon
 					arg_disable_filtering = true;
 					break;
 
+				case "--disable-on-battery":
+					arg_disable_on_battery = true;
+					break;
+
 				case "--allow-pattern":
 					if (next_arg == null)
 						break;
@@ -274,8 +283,15 @@ namespace Beagle.Daemon
 				}
 			}
 
-			// Set the IO priority so we don't slow down the system
-			IoPriority.ReduceIoPriority ();
+			if (SystemInformation.UsingBattery && arg_disable_on_battery) {
+				Log.Always ("Indexer is disabled when on battery power (--disable-on-battery)");
+				Environment.Exit (0);
+			}
+
+			// Set system priorities so we don't slow down the system
+			SystemPriorities.ReduceIoPriority ();
+			SystemPriorities.SetSchedulerPolicyBatch ();
+			SystemPriorities.Renice (19);
 			
 			driver = new LuceneIndexingDriver (arg_output, false);
 			driver.TextCache = (arg_cache_text) ? new TextCache (arg_output) : null;
@@ -451,7 +467,7 @@ namespace Beagle.Daemon
 
 		static Indexable FileToIndexable (FileInfo file)
 		{
-			if (!file.Exists || Ignore (file) || fa_store.IsUpToDate (file.FullName))
+			if (!file.Exists || Ignore (file) || fa_store.IsUpToDateAndFiltered (file.FullName))
 				return null;
 
 			// Create the indexable and add the standard properties we
@@ -479,7 +495,7 @@ namespace Beagle.Daemon
 
 			// If the directory exists in the fa store, then it is already indexed
 			if (attr != null) {
-				if (arg_delete && dir.LastWriteTimeUtc > attr.LastWriteTime)
+				if (arg_delete && ! FileAttributesStore.IsUpToDate (dir.FullName, attr))
 					modified_directories.Enqueue (dir);
 				return null;
 			}
