@@ -39,12 +39,10 @@ namespace Beagle.Daemon.GaimLogQueryable {
 	[QueryableFlavor (Name="GaimLog", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class GaimLogQueryable : LuceneFileQueryable, IIndexableGenerator {
 
-		private static Logger log = Logger.Get ("GaimLogQueryable");
-
 		private string config_dir, log_dir, icons_dir;
 
 		private int polling_interval_in_seconds = 60;
-		
+
 		private GaimBuddyListReader list = new GaimBuddyListReader ();
 
 		public GaimLogQueryable () : base ("GaimLogIndex")
@@ -63,7 +61,7 @@ namespace Beagle.Daemon.GaimLogQueryable {
 				return;
 			}
 
-			log.Info ("Starting Gaim log backend");
+			Log.Info ("Starting Gaim log backend");
 
 			Stopwatch stopwatch = new Stopwatch ();
 			stopwatch.Start ();
@@ -87,7 +85,7 @@ namespace Beagle.Daemon.GaimLogQueryable {
 
 			stopwatch.Stop ();
 
-			log.Info ("Gaim log backend worker thread done in {0}", stopwatch); 
+			Log.Info ("Gaim log backend worker thread done in {0}", stopwatch); 
 		}
 		
 		public override void Start () 
@@ -116,6 +114,8 @@ namespace Beagle.Daemon.GaimLogQueryable {
 
 		private void Crawl (bool index)
 		{
+			this.IsIndexing = true;
+
 			if (Inotify.Enabled)
 				Inotify.Subscribe (log_dir, OnInotifyNewProtocol, Inotify.EventType.Create);
 
@@ -155,6 +155,8 @@ namespace Beagle.Daemon.GaimLogQueryable {
 				foreach (FileInfo file in DirectoryWalker.GetFileInfos (remote_dir))
 					if (FileIsInteresting (file.Name))
 						IndexLog (file.FullName, Scheduler.Priority.Delayed);
+
+				IsIndexing = false;
 			}
 		}
 
@@ -173,7 +175,12 @@ namespace Beagle.Daemon.GaimLogQueryable {
 			if (log_files == null)
 				log_files = DirectoryWalker.GetFileInfosRecursive (log_dir).GetEnumerator ();
 
-			return log_files.MoveNext ();
+			if (log_files.MoveNext ())
+				return true;
+			else {
+				IsIndexing = false;
+				return false;
+			}
 		}
 
 		public Indexable GetNextIndexable ()
@@ -310,6 +317,14 @@ namespace Beagle.Daemon.GaimLogQueryable {
 
 		override protected bool HitFilter (Hit hit) 
 		{
+			// If the protocol isn't set (because maybe we got an
+			// exception while we were indexing), this isn't a
+			// valid hit.
+			if (hit ["fixme:protocol"] == null) {
+				Log.Warn ("Discarding IM log hit with missing protocol info: {0}", hit.Uri);
+				return false;
+			}
+
 			string speakingto = hit ["fixme:speakingto"];
 
 			// We have no idea who we're speaking to.  Bad, but we
