@@ -37,8 +37,6 @@ namespace Beagle.Daemon.KonqQueryable {
 	[QueryableFlavor (Name="KonquerorHistory", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class KonqQueryable : LuceneFileQueryable, IIndexableGenerator {
 
-		private static Logger log = Logger.Get ("KonqQueryable");
-
 		string konq_cache_dir;
 		private IEnumerator directory_enumerator = null;
 		private int polling_interval_in_seconds = 300; // 5 min
@@ -51,7 +49,15 @@ namespace Beagle.Daemon.KonqQueryable {
 			/* How to determine kio-http cache location ?
 			 * From KDE web-page it looks like /var/tmp/kdecache-$USERNAME/http
 			 */
-			konq_cache_dir = "/var/tmp/kdecache-" + System.Environment.UserName + "/http";
+			//Now we use the $KDEVARTMP env variable
+			string tmpdir = Environment.GetEnvironmentVariable ("KDEVARTMP");
+
+			if (tmpdir == null || tmpdir == "")
+    				tmpdir = "/var/tmp";
+
+			konq_cache_dir = Path.Combine (tmpdir, "kdecache-" + Environment.UserName ); 
+			konq_cache_dir = Path.Combine (konq_cache_dir, "http"); 
+			Log.Debug ("KonqCacheDir: " + konq_cache_dir);
 		}
 
 		/////////////////////////////////////////////////
@@ -82,18 +88,16 @@ namespace Beagle.Daemon.KonqQueryable {
 				ThisScheduler.Add (crawl_task);
 			}
 
-                        log.Info ("Starting Konq history backend ...");
+                        Log.Info ("Starting Konq history backend ...");
 			Crawl ();
 		}
 
 		private void Crawl ()
 		{
-                        State = QueryableState.Crawling;
 			directory_enumerator = DirectoryWalker.GetDirectoryInfos (konq_cache_dir).GetEnumerator ();
 			Scheduler.Task crawl_task = NewAddTask (this);
 			crawl_task.Tag = crawler_tag;
 			ThisScheduler.Add (crawl_task);
-			State = QueryableState.Idle;
 		}
 
 		private string crawler_tag = "Konqueror History Crawler";
@@ -167,46 +171,47 @@ namespace Beagle.Daemon.KonqQueryable {
 				return null;
 			}
 			
-			StreamReader reader = new StreamReader (stream, latin_encoding);
-			string url = null;
-			string creation_date = null;
-			string mimetype = null;
-			string charset = null;
-			bool is_ok = KonqHistoryUtil.ShouldIndex (reader, 
-										   out url, 
-										   out creation_date,
-										   out mimetype, 
-										   out charset);
+			using (StreamReader reader = new StreamReader (stream, latin_encoding)) {
+				string url = null;
+				string creation_date = null;
+				string mimetype = null;
+				string charset = null;
+				bool is_ok = KonqHistoryUtil.ShouldIndex (reader, 
+									  out url, 
+									  out creation_date,
+									  out mimetype, 
+									  out charset);
 			
-			if (!is_ok || url == String.Empty) {
-				//Logger.Log.Debug ("KonqQ: Skipping non-html file " + path + " of type=" + mimetype);
-				// finding out if a cache file should be indexed is expensive
-				// so, soon after we run the test, write lastwritetime attribute
-				FileAttributesStore.AttachLastWriteTime (path, DateTime.UtcNow);
-				return null; // we wont index bad files and non-html files
-			}
+				if (!is_ok || url == String.Empty) {
+					//Logger.Log.Debug ("KonqQ: Skipping non-html file " + path + " of type=" + mimetype);
+					// finding out if a cache file should be indexed is expensive
+					// so, soon after we run the test, write lastwritetime attribute
+					FileAttributesStore.AttachLastWriteTime (path, DateTime.UtcNow);
+					return null; // we wont index bad files and non-html files
+				}
 
-			Logger.Log.Debug ("KonqQ: Indexing " + path + " with url=" + url);
-			Uri uri = new Uri (url, true);
-			if (uri.Scheme == Uri.UriSchemeHttps) {
-				Logger.Log.Error ("Indexing secure https:// URIs is not secure!");
-				return null;
-			}
+				Logger.Log.Debug ("KonqQ: Indexing " + path + " with url=" + url);
+				Uri uri = new Uri (url, true);
+				if (uri.Scheme == Uri.UriSchemeHttps) {
+					Logger.Log.Error ("Indexing secure https:// URIs is not secure!");
+					return null;
+				}
 			
-			Indexable indexable = new Indexable (uri);
-			indexable.HitType = "WebHistory";
-			indexable.MimeType = KonqHistoryUtil.KonqCacheMimeType;
-			// store www.beaglewiki.org as www beagle org, till inpath: query is implemented
-			indexable.AddProperty (Property.NewUnstored ("fixme:urltoken", StringFu.UrlFuzzyDivide (url)));
-			// hint for the filter about the charset
-			indexable.AddProperty (Property.NewUnsearched (StringFu.UnindexedNamespace + "charset", charset));
+				Indexable indexable = new Indexable (uri);
+				indexable.HitType = "WebHistory";
+				indexable.MimeType = KonqHistoryUtil.KonqCacheMimeType;
+				// store www.beaglewiki.org as www beagle org, till inpath: query is implemented
+				indexable.AddProperty (Property.NewUnstored ("fixme:urltoken", StringFu.UrlFuzzyDivide (url)));
+				// hint for the filter about the charset
+				indexable.AddProperty (Property.NewUnsearched (StringFu.UnindexedNamespace + "charset", charset));
 			
-			DateTime date = new DateTime (1970, 1, 1);
-			date = date.AddSeconds (Int64.Parse (creation_date));
-			indexable.Timestamp = date;
+				DateTime date = DateTimeUtil.UnixToDateTimeUtc (0);
+				date = date.AddSeconds (Int64.Parse (creation_date));
+				indexable.Timestamp = date;
 
-			indexable.ContentUri = UriFu.PathToFileUri (path);
-			return indexable;
+				indexable.ContentUri = UriFu.PathToFileUri (path);
+				return indexable;
+			}
 		}
 
 		// FIXME: Implement removefile - removing files from history doesnt really make sense ? Do they ?
