@@ -13,9 +13,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
+
 namespace Lucene.Net.Search
 {
+	
+	
 	/// <summary> Expert: Returned by low-level sorted search implementations.
 	/// 
 	/// <p>Created: Feb 12, 2004 8:58:46 AM 
@@ -25,12 +29,12 @@ namespace Lucene.Net.Search
 	/// </author>
 	/// <since>   lucene 1.4
 	/// </since>
-	/// <version>  $Id: TopFieldDocs.cs,v 1.2 2005/10/06 19:29:57 dsd Exp $
+	/// <version>  $Id: TopFieldDocs.cs,v 1.3 2006/10/02 17:09:07 joeshaw Exp $
 	/// </version>
-	/// <seealso cref="Searchable#Search(Query,Filter,int,Sort)">
+	/// <seealso cref="Searcher.Search(Query,Filter,int,Sort)">
 	/// </seealso>
 	[Serializable]
-	public class TopFieldDocs:TopDocs
+	public class TopFieldDocs : TopDocs
 	{
 		
 		/// <summary>The fields which were used to sort results by. </summary>
@@ -43,7 +47,9 @@ namespace Lucene.Net.Search
 		/// </param>
 		/// <param name="fields">    The sort criteria used to find the top hits.
 		/// </param>
-		internal TopFieldDocs(int totalHits, ScoreDoc[] scoreDocs, SortField[] fields):base(totalHits, scoreDocs)
+		/// <param name="maxScore">  The maximum score encountered.
+		/// </param>
+		internal TopFieldDocs(int totalHits, ScoreDoc[] scoreDocs, SortField[] fields, float maxScore) : base(totalHits, scoreDocs, maxScore)
 		{
 			this.fields = fields;
 		}
