@@ -77,7 +77,8 @@ namespace Beagle.Daemon {
 		// 15: analyze PropertyKeyword field, and store all properties as
 		//     lower case so that we're truly case insensitive.
 		// 16: add inverted timestamp to make querying substantially faster
-		private const int MAJOR_VERSION = 16;
+		// 17: add boolean property to denote a child indexable
+		private const int MAJOR_VERSION = 17;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -106,7 +107,7 @@ namespace Beagle.Daemon {
 
 		//////////////////////////////////////////////////////////////////////////////
 
-		protected string IndexName { get { return index_name; } }
+		public string IndexName { get { return index_name; } }
 
 		public Lucene.Net.Store.Directory PrimaryStore { get { return primary_store; } }
 
@@ -375,7 +376,7 @@ namespace Beagle.Daemon {
 		}
 
 		// FIXME: This assumes everything being indexed is in English!
-		private class BeagleAnalyzer : StandardAnalyzer {
+		internal class BeagleAnalyzer : StandardAnalyzer {
 
 			private char [] buffer = new char [2];
 			private bool strip_extra_property_info = false;
@@ -420,7 +421,7 @@ namespace Beagle.Daemon {
 							
 						TokenStream singleton_stream = new SingletonTokenStream (reader.ReadToEnd ());
 						
-						if (fieldName.StartsWith ("prop:k:" + LuceneQueryingDriver.PrivateNamespace))
+						if (fieldName.StartsWith ("prop:k:" + Property.PrivateNamespace))
 							return singleton_stream;
 						else
 							return new LowerCaseFilter (singleton_stream);
@@ -484,7 +485,18 @@ namespace Beagle.Daemon {
 			case PropertyType.Date:    return "PropertyDate";
 			}
 
-			return null;
+			throw new Exception ("Bad property type: " + type);
+		}
+
+		static private Field.Index TypeToIndexInstruction (PropertyType type)
+		{
+			switch (type) {
+			case PropertyType.Text:    return Field.Index.TOKENIZED; // Full analysis
+			case PropertyType.Keyword: return Field.Index.TOKENIZED; // Lowercases keywords
+			case PropertyType.Date:    return Field.Index.NO_NORMS;  // Do nothing
+			}
+
+			throw new Exception ("Bad property type: " + type);
 		}
 
 		// Exposing this is a little bit suspicious.
@@ -501,16 +513,14 @@ namespace Beagle.Daemon {
 			Field f;
 			f = new Field ("YM:" + field_name,
 				       StringFu.DateTimeToYearMonthString (dt),
-				       false,   // never store
-				       true,    // always index
-				       false);  // never tokenize
+				       Field.Store.NO,
+				       Field.Index.NO_NORMS);
 			doc.Add (f);
 
 			f = new Field ("D:" + field_name,
 				       StringFu.DateTimeToDayString (dt),
-				       false,   // never store
-				       true,    // always index
-				       false);  // never tokenize
+				       Field.Store.NO,
+				       Field.Index.NO_NORMS);
 			doc.Add (f);
 		}
 
@@ -528,17 +538,21 @@ namespace Beagle.Daemon {
 
 			if (prop.IsSearched) {
 				string wildcard_field = TypeToWildcardField (prop.Type);
-				if (wildcard_field != null) {
-					f = new Field (wildcard_field,
-						       prop.Value,
-						       false, // never stored
-						       true,  // always indexed
-						       true); // always tokenize (just lowercases for keywords; full analysis for text)
-					doc.Add (f);
 
-					if (prop.Type == PropertyType.Date)
-						AddDateFields (wildcard_field, prop, doc);
-				}
+				f = new Field (wildcard_field,
+					       prop.Value,
+					       Field.Store.NO,
+					       TypeToIndexInstruction (prop.Type));
+
+				// We don't want to include norms for non-text
+				// fields, even if we do tokenize them.
+				if (prop.Type == PropertyType.Keyword || prop.Type == PropertyType.Date)
+					f.SetOmitNorms (true);
+
+				doc.Add (f);
+
+				if (prop.Type == PropertyType.Date)
+					AddDateFields (wildcard_field, prop, doc);
 			}
 
 			string coded_value;
@@ -550,9 +564,8 @@ namespace Beagle.Daemon {
 
 			f = new Field (field_name,
 				       coded_value,
-				       prop.IsStored,
-				       true,        // always index
-				       true);       // always tokenize (strips off type code for keywords and lowercases)
+				       prop.IsStored ? Field.Store.YES : Field.Store.NO,
+				       Field.Index.TOKENIZED);
 			doc.Add (f);
 
 			if (prop.Type == PropertyType.Date)
@@ -605,11 +618,13 @@ namespace Beagle.Daemon {
 
 			Field f;
 
-			f = Field.Keyword ("Uri", UriFu.UriToEscapedString (indexable.Uri));
+			f = new Field ("Uri", UriFu.UriToEscapedString (indexable.Uri),
+				       Field.Store.YES, Field.Index.NO_NORMS);
 			primary_doc.Add (f);
 
 			if (indexable.ParentUri != null) {
-				f = Field.Keyword ("ParentUri", UriFu.UriToEscapedString (indexable.ParentUri));
+				f = new Field ("ParentUri", UriFu.UriToEscapedString (indexable.ParentUri),
+					       Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 			}
 			
@@ -622,27 +637,30 @@ namespace Beagle.Daemon {
 				string wildcard_field = TypeToWildcardField (PropertyType.Date);
 
 				string str = StringFu.DateTimeToString (indexable.Timestamp);
-				f = Field.Keyword ("Timestamp", str);
+				f = new Field ("Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = Field.UnStored (wildcard_field, str);
+				f = new Field (wildcard_field, str, Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				// Create an inverted timestamp so that we can
 				// sort by timestamp at search-time.
 				long timeval = Convert.ToInt64 (str);
-				f = Field.UnStored ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString ());
+				f = new Field ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString (),
+					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToYearMonthString (indexable.Timestamp);
-				f = Field.Keyword ("YM:Timestamp", str);
+				f = new Field ("YM:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = Field.UnStored ("YM:" + wildcard_field, str);
+				f = new Field ("YM:" + wildcard_field, str,
+					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToDayString (indexable.Timestamp);
-				f = Field.Keyword ("D:Timestamp", str);
+				f = new Field ("D:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = Field.UnStored ("D:" + wildcard_field, str);
+				f = new Field ("D:" + wildcard_field, str,
+					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 			}
 
@@ -660,15 +678,18 @@ namespace Beagle.Daemon {
 
 				TextReader reader;
 				
+				// Add the field "Text" first
+				// It is important that the order is preserved
 				reader = indexable.GetTextReader ();
 				if (reader != null) {
-					f = Field.Text ("Text", reader);
+					f = new Field ("Text", reader);
 					primary_doc.Add (f);
 				}
 			
+				// Then add "HotText"
 				reader = indexable.GetHotTextReader ();
 				if (reader != null) {
-					f = Field.Text ("HotText", reader);
+					f = new Field ("HotText", reader);
 					primary_doc.Add (f);
 				}
 			}
@@ -693,16 +714,20 @@ namespace Beagle.Daemon {
 				AddPropertyToDocument (prop, primary_doc);
 			}
 
+			{
+				Property prop;
+				prop = Property.NewBool (Property.IsChildPropKey, indexable.IsChild);
+				AddPropertyToDocument (prop, primary_doc);
+			}
+
 			// Store the other properties
 				
 			foreach (Property prop in indexable.Properties) {
 				Document target_doc = primary_doc;
 				if (prop.IsMutable) {
-					if (secondary_doc == null) {
-						secondary_doc = new Document ();
-						f = Field.Keyword ("Uri", UriFu.UriToEscapedString (indexable.Uri));
-						secondary_doc.Add (f);
-					}
+					if (secondary_doc == null)
+						secondary_doc = CreateSecondaryDocument (indexable.Uri, indexable.ParentUri);
+
 					target_doc = secondary_doc;
 				}
 					
@@ -710,6 +735,22 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		static private Document CreateSecondaryDocument (Uri uri, Uri parent_uri)
+		{
+			Document secondary_doc = new Document ();
+
+			Field f = new Field ("Uri", UriFu.UriToEscapedString (uri), Field.Store.YES, Field.Index.NO_NORMS);
+			secondary_doc.Add (f);
+			
+			if (parent_uri != null) {
+				// Store both Uri and ParentUri in secondary index for easy removal
+				f = new Field ("ParentUri", UriFu.UriToEscapedString (parent_uri), Field.Store.YES, Field.Index.NO_NORMS);
+				secondary_doc.Add (f);
+			}
+
+			return secondary_doc;
+		}
+
 		static protected Document RewriteDocument (Document old_secondary_doc,
 							   Indexable prop_only_indexable)
 		{
@@ -720,11 +761,17 @@ namespace Beagle.Daemon {
 			new_doc = new Document ();
 
 			Field uri_f;
-			uri_f = Field.Keyword ("Uri", UriFu.UriToEscapedString (prop_only_indexable.Uri));
+			uri_f = new Field ("Uri", UriFu.UriToEscapedString (prop_only_indexable.Uri), Field.Store.YES, Field.Index.NO_NORMS);
 			new_doc.Add (uri_f);
 
 			Logger.Log.Debug ("Rewriting {0}", prop_only_indexable.DisplayUri);
 
+			if (prop_only_indexable.ParentUri != null) {
+				uri_f = new Field ("ParentUri", UriFu.UriToEscapedString (prop_only_indexable.ParentUri), Field.Store.YES, Field.Index.NO_NORMS);
+				new_doc.Add (uri_f);
+				Logger.Log.Debug ("Parent Uri {0}", prop_only_indexable.ParentUri);
+			}
+
 			// Add the new properties to the new document.  To
 			// delete a property, set the Value to null... then it
 			// will be added to seen_props (so the old value will
@@ -986,7 +1033,7 @@ namespace Beagle.Daemon {
 		// YM(Jun 2005, Sep 2006) OR
 		// (YM(Oct 2006) AND D(1,16))
 
-		static private DateTime lower_bound = new DateTime (1970, 1, 1);
+		static private DateTime lower_bound = DateTimeUtil.UnixToDateTimeUtc (0);
 
 		// FIXME: we should probably boost this sometime around 2030.
 		// Mark your calendar.
@@ -1213,16 +1260,22 @@ namespace Beagle.Daemon {
 				LNS.BooleanQuery p_query = new LNS.BooleanQuery ();
 				LNS.BooleanQuery s_query = new LNS.BooleanQuery ();
 
+				bool added_subquery = false;
+
 				if (part.SearchFullText) {
 					LNS.Query subquery;
 					subquery = StringToQuery ("Text", part.Text, term_list);
-					if (subquery != null)
+					if (subquery != null) {
 						p_query.Add (subquery, false, false);
+						added_subquery = true;
+					}
 
 					// FIXME: HotText is ignored for now!
 					// subquery = StringToQuery ("HotText", part.Text);
-					// if (subquery != null) 
+					// if (subquery != null) {
 					//    p_query.Add (subquery, false, false);
+					//    added_subquery = true;
+					// }
 				}
 
 				if (part.SearchTextProperties) {
@@ -1233,20 +1286,45 @@ namespace Beagle.Daemon {
 						// Properties can live in either index
 						if (! only_build_primary_query)
 							s_query.Add (subquery.Clone () as LNS.Query, false, false);
+						added_subquery = true;
 					}
 
-					Term term;
-					term = new Term ("PropertyKeyword", part.Text.ToLower ()); // make sure text is lowercased
-					// FIXME: terms are already added in term_list. But they may have been tokenized
-					// The term here is non-tokenized version. Should this be added to term_list ?
-					// term_list is used to calculate scores
-					if (term_list != null)
-						term_list.Add (term);
-					subquery = new LNS.TermQuery (term);
-					p_query.Add (subquery, false, false);
-					// Properties can live in either index
-					if (! only_build_primary_query)
-						s_query.Add (subquery.Clone () as LNS.Query, false, false);
+					// The "added_subquery" check is to handle the situation where
+					// a part of the text is a stop word.  Normally, a search for
+					// "hello world" would break down into this query:
+					//
+					// (Text:hello OR PropertyText:hello OR PropertyKeyword:hello)
+					// AND (Text:world OR PropertText:world OR PropertyKeyword:world)
+					//
+					// This fails with stop words, though.  Let's assume that "world"
+					// is a stop word.  You would end up with:
+					//
+					// (Text:hello OR PropertyText:hello OR PropertyKeyword:hello)
+					// AND (PropertyKeyword:world)
+					//
+					// Which is not what we want.  We'd want to match documents that
+					// had only "hello" without also having a keyword "world".  In
+					// this case, don't create the PropertyKeyword part of the query,
+					// since it would be included in the larger set if it weren't
+					// required anyway.
+					if (added_subquery) {
+						Term term;
+						term = new Term ("PropertyKeyword", part.Text.ToLower ()); // make sure text is lowercased
+						// FIXME: terms are already added in term_list. But they may have been tokenized
+						// The term here is non-tokenized version. Should this be added to term_list ?
+						// term_list is used to calculate scores
+						if (term_list != null)
+							term_list.Add (term);
+						subquery = new LNS.TermQuery (term);
+						p_query.Add (subquery, false, false);
+						// Properties can live in either index
+						if (! only_build_primary_query)
+							s_query.Add (subquery.Clone () as LNS.Query, false, false);
+					} else {
+						// Reset these so we return a null query
+						p_query = null;
+						s_query = null;
+					}
 				}
 
 				primary_query = p_query;
@@ -1303,12 +1381,9 @@ namespace Beagle.Daemon {
 				QueryPart_Property part = (QueryPart_Property) abstract_part;
 
 				string field_name;
-				if (part.Key == QueryPart_Property.AllProperties) {
+				if (part.Key == QueryPart_Property.AllProperties)
 					field_name = TypeToWildcardField (part.Type);
-					// FIXME: probably shouldn't just return silently
-					if (field_name == null)
-						return;
-				} else
+				else
 					field_name = PropertyToFieldName (part.Type, part.Key);
 
 				if (part.Type == PropertyType.Text)
@@ -1332,17 +1407,16 @@ namespace Beagle.Daemon {
 
 				QueryPart_DateRange part = (QueryPart_DateRange) abstract_part;
 
+				// FIXME: We don't handle prohibited queries with sub-date
+				// accuracy.  For example, if we say we prohibit matches
+				// between 5 May 2007 at 2 PM and 8 May at 5 AM, we'll
+				// miss any matches that happen between midnight and 2 PM
+				// on 5 May 2007 and between midnight and 5 AM on 8 May.
+
 				primary_query = GetDateRangeQuery (part, out hit_filter);
 				// Date properties can live in either index
 				if (! only_build_primary_query && primary_query != null)
 					secondary_query = primary_query.Clone () as LNS.Query;
-
-				// If this is a prohibited part, invert our hit filter.
-				if (part.Logic == QueryPartLogic.Prohibited) {
-					NotHitFilter nhf;
-					nhf = new NotHitFilter (hit_filter);
-					hit_filter = new HitFilter (nhf.HitFilter);
-				}
 				
 				return;
 			}
@@ -1522,7 +1596,6 @@ namespace Beagle.Daemon {
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
-					rav.Refcount++;
 
 					directory_rav_map [directory] = rav;
 					reader_rav_map [reader] = rav;
@@ -1533,17 +1606,15 @@ namespace Beagle.Daemon {
 				version = IndexReader.GetCurrentVersion (directory);
 				
 				if (version != rav.Version) {
-					UnrefReaderAndVersion_Unlocked (rav);
-
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
-					rav.Refcount++;
 
 					directory_rav_map [directory] = rav;
 					reader_rav_map [reader] = rav;
-				} else
+				} else {
 					rav.Refcount++;
+				}
 
 				return rav.Reader;
 			}
@@ -1554,8 +1625,9 @@ namespace Beagle.Daemon {
 			rav.Refcount--;
 
 			if (rav.Refcount == 0) {
-				rav.Reader.Close ();
 				reader_rav_map.Remove (rav.Reader);
+				directory_rav_map.Remove (rav.Reader.Directory ());
+				rav.Reader.Close ();
 			}
 		}
 
@@ -1564,7 +1636,10 @@ namespace Beagle.Daemon {
 			lock (reader_rav_map) {
 				ReaderAndVersion rav = (ReaderAndVersion) reader_rav_map [reader];
 
-				UnrefReaderAndVersion_Unlocked (rav);
+				if (rav != null)
+					UnrefReaderAndVersion_Unlocked (rav);
+				else
+					reader.Close ();
 			}
 		}
 
@@ -1734,5 +1809,46 @@ namespace Beagle.Daemon {
 
 			return all_hits;
 		}
+
+		// For a large index, this will be very slow and will consume
+		// a lot of memory.  Don't call it without a good reason!
+		public ICollection GetHitsForUris (ICollection uris)
+		{
+			Hashtable hits_by_uri = UriFu.NewHashtable ();
+
+			LNS.IndexSearcher primary_searcher = GetSearcher (PrimaryStore);
+			LNS.IndexSearcher secondary_searcher = GetSearcher (SecondaryStore);
+
+			LNS.Query uri_query = UriQuery ("Uri", uris);
+
+			LNS.Hits primary_hits = primary_searcher.Search (uri_query);
+
+			for (int i = 0; i < primary_hits.Length (); i++) {
+				Document doc = primary_hits.Doc (i);
+
+				Uri u = GetUriFromDocument (doc);
+
+				Hit hit = DocumentToHit (doc);
+				hits_by_uri [hit.Uri] = hit;
+			}
+
+			if (secondary_searcher != null) {
+				LNS.Hits secondary_hits = secondary_searcher.Search (uri_query);
+
+				for (int i = 0; i < secondary_hits.Length (); i++) {
+					Document doc = secondary_hits.Doc (i);
+
+					Uri uri = GetUriFromDocument (doc);
+					Hit hit = (Hit) hits_by_uri [uri];
+					if (hit != null)
+						AddPropertiesToHit (hit, doc, false);
+				}
+			}
+
+			ReleaseSearcher (primary_searcher);
+			ReleaseSearcher (secondary_searcher);
+
+			return hits_by_uri.Values;
+		}
 	}
 }
