@@ -24,18 +24,22 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-//
-// This queryable just takes the LuceneQueryable and adds some sane
-// default behavior for indexing files.
-//
-
 using System;
 using System.Collections;
+using System.IO;
 
 using Beagle.Util;
 
 namespace Beagle.Daemon {
 
+	/**
+	 * This queryable just takes the LuceneQueryable and adds some sane
+	 * default behavior for indexing files containing multiple indexables.
+	 * Suitable for feedfiles or mbox style mail files.
+	 * Use this only if the backend generates multiple indexables from one
+	 * physical source file; _do not_ use this if multiple indexables
+	 * are created from one indexable while filtering (aka child indexables).
+	 */
 	public abstract class LuceneFileQueryable : LuceneQueryable {
 
 		public LuceneFileQueryable (string index_name, int minor_version, bool disable_locking) :
@@ -98,8 +102,10 @@ namespace Beagle.Daemon {
 			if (indexable.Type == IndexableType.Remove)
 				return true;
 
-			// Remember the file's info and mtime.
-			CachedFileInfo info = new CachedFileInfo ();
+			CachedFileInfo info = (CachedFileInfo) file_info_cache [indexable.Uri];
+
+			if (info == null)
+				info = new CachedFileInfo ();
 
 			info.Uri = indexable.Uri;
 
@@ -113,14 +119,20 @@ namespace Beagle.Daemon {
 				IncrementReferenceCount (info.Path);
 			}
 
-			try {
-				info.Mtime = FileSystem.GetLastWriteTimeUtc (info.Path);
-			} catch (Exception ex) {
+			// The path could be null in certain cases:
+			//    * The indexable is a non-file URI and no
+			//      parent URI is set.
+			//    * The indexable is a child indexable and the
+			//      parent URI is not a file URI.
+			if (info.Path == null)
+				return true;
+
+			info.Mtime = FileSystem.GetLastWriteTimeUtc (info.Path);
+
+			if (! FileSystem.ExistsByDateTime (info.Mtime)) {
 				// If we can't get an mtime for the file, it must
 				// have disappeared out from under us.  In that case,
 				// don't bother adding anything.
-				// FIXME: Should we be more specific and only catch
-				// a file not found error here?
 				return false;
 			}
 
@@ -137,6 +149,8 @@ namespace Beagle.Daemon {
 			if (info == null)
 				return;
 
+			file_info_cache.Remove (info.Uri);
+
 			// Yeah, this is ghetto. If it's a file that's shared across multiple
 			// indexables, only tag it with when the last indexable has been indexed.
 			if (info.Shared && DecrementReferenceCount (info.Path))
@@ -159,7 +173,7 @@ namespace Beagle.Daemon {
 			}
 
 			if (! FileAttributesStore.Write (attr))
-				Logger.Log.Error ("Couldn't write attributes for {0}", info.Path);
+				Logger.Log.Warn ("Couldn't write attributes for {0}", info.Path);
 		}
 
 		override protected void PostRemoveHook (Indexable indexable, IndexerRemovedReceipt receipt)
