@@ -77,10 +77,8 @@ namespace Beagle.Daemon.ThunderbirdQueryable {
 				return;
 			}
 			
-			State = QueryableState.Crawling;
 			indexer = new ThunderbirdIndexer (this, Thunderbird.GetProfilePaths (root_path));
 			indexer.Crawl ();
-			State = QueryableState.Idle;
 			
 			watch.Stop ();
 			Logger.Log.Info ("Thunderbird backend done in {0}s", watch.ElapsedTime);
